@@ -2,7 +2,7 @@ from __future__ import unicode_literals
 
 import py
 from devpi_common.metadata import (sorted_sameproject_links,
-                                   get_latest_version, BasenameMeta)
+                                   get_latest_version)
 from devpi_common.validation import validate_metadata, normalize_name
 from devpi_common.types import ensure_unicode
 from devpi_common.archive import Archive
@@ -390,8 +390,8 @@ class PrivateStage:
             projectconfig = self.get_projectconfig(name)
             return projectconfig.get(version)
 
-    def get_metadata_latest(self, name):
-        versions = self.get_projectconfig(name)
+    def get_metadata_latest_perstage(self, name):
+        versions = self.get_projectconfig_perstage(name)
         maxver = get_latest_version(versions)
         return self.get_metadata(name, maxver.string)
 
@@ -475,16 +475,14 @@ class PrivateStage:
     class MissesRegistration(Exception):
         """ store_releasefile requires pre-existing release metadata. """
 
-    def store_releasefile(self, filename, content, last_modified=None):
+    def store_releasefile(self, name, version, filename, content,
+                          last_modified=None):
         filename = ensure_unicode(filename)
-        bmeta = BasenameMeta(filename)
-        name, version = bmeta.name, bmeta.version
-        info = self.get_project_info(name)
-        name = getattr(info, "name", name)
         if not self.get_metadata(name, version):
             raise self.MissesRegistration(name, version)
         log.debug("project name of %r is %r", filename, name)
-        key = self.keyfs.PROJCONFIG(user=self.user, index=self.index, name=name)
+        key = self.keyfs.PROJCONFIG(user=self.user,
+                                    index=self.index, name=name)
         with key.locked_update() as projectconfig:
             verdata = projectconfig.setdefault(version, {})
             files = verdata.setdefault("+files", {})
@@ -500,8 +498,9 @@ class PrivateStage:
         """ store zip file and unzip doc content for the
         specified "name" project. """
         if not version:
-            version = self.get_metadata_latest(name)["version"]
-            log.info("store_doczip: derived version %s", version)
+            version = self.get_metadata_latest_perstage(name)["version"]
+            log.info("store_doczip: derived version of %s is %s",
+                     name, version)
         key = self.keyfs.PROJCONFIG(user=self.user, index=self.index, name=name)
         with key.locked_update() as projectconfig:
             verdata = projectconfig[version]
