@@ -3,27 +3,36 @@ Module for handling storage and proxy-streaming and caching of release files
 for all indexes.
 
 """
+from __future__ import unicode_literals
 import hashlib
 import posixpath
 import os
+import sys
 import json
 from wsgiref.handlers import format_date_time
 from datetime import datetime
 from time import mktime
 
 import py
-from .types import propmapping
-from .urlutil import DistURL, splitbasename
+from devpi_common.types import propmapping, ensure_unicode
 
 from logging import getLogger
 log = getLogger(__name__)
 
 class FileStore:
+    attachment_encoding = "utf-8"
+
     def __init__(self, keyfs):
         self.keyfs = keyfs
 
     def maplink(self, link, refresh=False):
-        key = self.keyfs.PYPIFILES(relpath=link.torelpath())
+        if link.md5:
+            key = self.keyfs.STAGEFILE(user="root", index="pypi",
+                                       md5=link.md5 or "unknown_md5",
+                                       filename=link.basename)
+        else:
+            key = self.keyfs.PYPIFILE_NOMD5(user="root", index="pypi",
+                   relpath=link.torelpath())
         entry = self.getentry(key.relpath)
         mapping = {"url": link.geturl_nofragment().url}
         mapping["eggfragment"] = link.eggfragment
@@ -114,40 +123,52 @@ class FileStore:
         return entry.gethttpheaders(), iter_and_cache()
 
     def store(self, user, index, filename, content, last_modified=None):
-        md5 = getmd5(content)
-        proj, version = splitbasename(filename)[:2]
+        return self.store_file(user, index, filename, py.io.BytesIO(content),
+                               last_modified=last_modified)
+
+    def store_file(self, user, index, filename, fil, last_modified=None,
+                   chunksize=524288):
+        hash = hashlib.md5()
+        with self.keyfs.tempfile() as w:
+            size = 0
+            while 1:
+                s = fil.read(chunksize)
+                if not s:
+                    break
+                hash.update(s)
+                size += len(s)
+                w.write(s)
+
+        digest = hash.hexdigest()
         key = self.keyfs.STAGEFILE(user=user, index=index,
-                                   #md5=md5,
-                                   proj=proj, version=version,
-                                   filename=filename)
+                                   md5=digest, filename=filename)
+        self.keyfs._rename(w.key.relpath, key.relpath)
         entry = self.getentry(key.relpath)
-        key.set(content)
         if last_modified is None:
             last_modified = http_date()
-        entry.set(md5=md5, size=len(content),
-                  last_modified=last_modified)
+        entry.set(md5=digest, size=size, last_modified=last_modified)
         return entry
 
     def add_attachment(self, md5, type, data):
         assert type in ("toxresult",)
-        assert isinstance(data, bytes)
         # XXX thread safety
         num = len(self.keyfs.ATTACHMENT.listnames("num",
                                                   type=type,
                                                   md5=md5))
         num = str(num)
         key = self.keyfs.ATTACHMENT(type=type, md5=md5, num=num)
-        key.set(data)
+        key.set(data.encode(self.attachment_encoding))
         return num
 
     def get_attachment(self, md5, type, num):
-        return self.keyfs.ATTACHMENT(type=type, md5=md5, num=num).get()
+        data = self.keyfs.ATTACHMENT(type=type, md5=md5, num=num).get()
+        return data.decode(self.attachment_encoding)
 
     def iter_attachments(self, md5, type):
         nums = self.keyfs.ATTACHMENT.listnames("num", type=type, md5=md5)
         for num in range(len(nums)):
             a = self.keyfs.ATTACHMENT(num=str(num), type=type, md5=md5).get()
-            yield json.loads(a)
+            yield json.loads(a.decode(self.attachment_encoding))
 
     def iter_attachment_types(self, md5):
         return self.keyfs.ATTACHMENT.listnames("type", md5=md5, num="0")
@@ -171,6 +192,10 @@ class RelPathEntry(object):
         #log.debug("self.FILE %s", self.FILE)
         self._mapping = self.PATHENTRY.get()
 
+    @property
+    def filepath(self):
+        return self.FILE.filepath
+
     def __repr__(self):
         return "<RelPathEntry %r>" %(self.relpath)
 
@@ -221,14 +246,14 @@ class RelPathEntry(object):
         for name, val in kw.items():
             assert name in self._attr
             if val is not None:
-                mapping[name] = str(val)
+                mapping[name] = "%s" % (val,)
         self._mapping.update(mapping)
-
         self.PATHENTRY.set(self._mapping)
 
 for _ in RelPathEntry._attr:
-    setattr(RelPathEntry, _, propmapping(_))
-
+    if sys.version_info < (3,0):
+        _ = _.encode("ascii")  # py2 needs str (bytes)
+    setattr(RelPathEntry, _, propmapping(_, convert=ensure_unicode))
 
 def http_date():
     now = datetime.now()
