@@ -1,9 +1,9 @@
-
+from __future__ import unicode_literals
 import py
 import sys
 import os
 import subprocess
-from devpi_server import urlutil
+from devpi_common.url import urlparse
 
 from devpi_server.config import render, getpath
 import devpi_server
@@ -45,7 +45,7 @@ def gendeploycfg(config, venvdir, tw=None):
         outside_host = "localhost"
         outside_port = 80
     else:
-        parsed = urlutil.urlparse(outside_url)
+        parsed = urlparse(outside_url)
         parts = list(parsed.netloc.split(":"))
         if len(parts) < 2:
             parts.append(80)
@@ -134,7 +134,6 @@ def gendeploy(config):
         return 1
     target = getpath(config.args.gendeploy)
     devpi_ctl = target.join("bin", "devpi-ctl")
-    prefix = ""
     if devpi_ctl.check():
         tw.line("detected existing devpi-ctl, ensuring it is shut down",
                 red=True)
