@@ -4,17 +4,17 @@ filesystem key/value storage with support for
 storing basic python types.
 
 """
+from __future__ import unicode_literals
 
-import re
 import py
 import threading
-from tempfile import NamedTemporaryFile, mkdtemp
+from tempfile import NamedTemporaryFile
 from execnet import dumps, loads
-import os, sys
+import os
+import sys
 from os.path import basename, isabs, join
-from os import listdir
 
-from devpi_server.types import cached_property
+from devpi_common.types import cached_property
 
 _nodefault = object()
 
@@ -37,7 +37,7 @@ class KeyFS(object):
         try:
             with path.open("rb") as f:
                 return f.read()
-        except py.error.ENOENT:
+        except py.error.Error:
             if default is _nodefault:
                 raise KeyError(relpath)
             return default
@@ -46,10 +46,10 @@ class KeyFS(object):
         if self._mode is None:
             umask = os.umask(0)
             os.umask(umask)
-            self._mode = 0666 ^ umask
+            self._mode = 0o666 ^ umask
         os.chmod(file_name, self._mode)
 
-    def tempfile(self, prefix):
+    def tempfile(self, prefix="tmp"):
         f = NamedTemporaryFile(prefix=prefix, dir=self.tmpdir, delete=False)
         relpath = os.path.relpath(f.name, str(self.basedir))
         # change from hardcoded default perm 0600 in tempfile._mkstemp_inner()
@@ -58,9 +58,8 @@ class KeyFS(object):
         return f
 
     def mkdtemp(self, prefix):
-        return py.path.local(mkdtemp(prefix=prefix, dir=self.tmpdir))
-        #return get_typed_key(self, py.path.local(path).relto(self.basedir),
-        #                     "DIR")
+        return py.path.local.make_numbered_dir(
+                    prefix=prefix, rootdir=py.path.local(self.tmpdir))
 
     def _set(self, relpath, value):
         with self.tempfile(basename(relpath)) as f:
@@ -68,14 +67,14 @@ class KeyFS(object):
         self._rename(f.key.relpath, relpath)
         return True
 
-    def _rename(self, sourcekey, destkey):
-        assert not isabs(sourcekey), sourcekey
-        assert not isabs(destkey), destkey
-        source = join(str(self.basedir), sourcekey)
-        dest = join(str(self.basedir), destkey)
+    def _rename(self, rel_source, rel_dest):
+        assert not isabs(rel_source), rel_source
+        assert not isabs(rel_dest), rel_dest
+        source = join(str(self.basedir), rel_source)
+        dest = join(str(self.basedir), rel_dest)
         try:
             os.rename(source, dest)
-        except OSError as e:
+        except OSError:
             destdir = os.path.dirname(dest)
             if not os.path.exists(destdir):
                 os.makedirs(destdir)
@@ -96,7 +95,8 @@ class KeyFS(object):
         return path.check()
 
     def _getpath(self, relpath):
-        assert isinstance(relpath, (str, unicode)), (type(relpath), relpath)
+        assert isinstance(relpath, py.builtin._basestring), \
+               (type(relpath), relpath)
         return self.basedir.join(relpath)
 
     def destroyall(self):
@@ -104,6 +104,7 @@ class KeyFS(object):
             self.basedir.remove()
 
     def addkey(self, key, type):
+        assert isinstance(key, py.builtin._basestring)
         if "{" in key:
             key = PTypedKey(self, key, type)
         else:
@@ -115,18 +116,11 @@ class KeyFS(object):
 class PTypedKey:
     def __init__(self, keyfs, key, type):
         self.keyfs = keyfs
-        self.key = key
+        self.key = py.builtin._totext(key)
         self.type = type
 
     def __call__(self, **kw):
-        newkw = {}
-        for name, val in kw.items():
-            if py.builtin._istext(val):
-                val = val.encode("utf8")
-            else:
-                assert py.builtin._isbytes(val), (name, val)
-            newkw[name] = val
-        realkey = self.key.format(**newkw)
+        realkey = self.key.format(**kw)
         return get_typed_key(self.keyfs, realkey, self.type)
 
     def listnames(self, __name, **kw):
@@ -156,7 +150,7 @@ class PTypedKey:
         for x in current:
             key = x.relto(self.keyfs.basedir)
             parts = key.split(os.sep)
-            names.add(parts[position])
+            names.add(py.builtin._totext(parts[position]))
         return names
 
     def __repr__(self):
