@@ -12,12 +12,11 @@ from logging import getLogger
 log = getLogger(__name__)
 
 from devpi_common.types import cached_property
-from devpi_common.metadata import Version
 from devpi_common.request import new_requests_session
 from .config import parseoptions, configure_logging
+from .extpypi import XMLProxy
+from . import __version__ as server_version
 
-import devpi_server
-server_version = devpi_server.__version__
 
 PYPIURL_XMLRPC = "https://pypi.python.org/pypi/"
 
@@ -32,10 +31,21 @@ def check_compatible_version(xom):
     if args.upgrade_state or args.export:
         return
     state_version = xom.get_state_version()
-    if Version(server_version) != Version(state_version):
-        fatal("Incompatible state: server %s cannot run serverdir "
-              "%s created by %s. " %(server_version, xom.config.serverdir,
-                             state_version))
+    if server_version != state_version:
+        state_ver = map(int, state_version.split(".")[:2])
+        server_ver = map(int, server_version.split(".")[:2])
+        if state_ver != server_ver:
+            fatal("Incompatible state: server %s cannot run serverdir "
+                  "%s created by %s. "
+                  "Use --export from older version, then --import with newer "
+                  "version.  Or try --upgrade-state for in-place upgrades."
+                  " But do a backup first :)"
+                  %(server_version, xom.config.serverdir, state_version))
+
+        xom.set_state_version(server_version)
+        tw = py.io.TerminalWriter(sys.stderr)
+        tw.line("minor version upgrade: setting serverstate to %s from %s" %(
+                server_version, state_version), bold=True)
 
 def main(argv=None):
     """ devpi-server command line entry point. """
@@ -55,7 +65,7 @@ def _main(argv=None):
     args = config.args
 
     if args.version:
-        print(devpi_server.__version__)
+        print(server_version)
         return
 
     # meta commmands (that will not instantiate server object in-process)
@@ -262,13 +272,7 @@ class XOM:
 
     @cached_property
     def proxy(self):
-        from devpi_server.extpypi import XMLProxy
-        try:
-            from xmlrpc.client import ServerProxy
-        except ImportError:
-            # PY2
-            from xmlrpclib import ServerProxy
-        return XMLProxy(ServerProxy(PYPIURL_XMLRPC))
+        return XMLProxy(PYPIURL_XMLRPC)
 
     @cached_property
     def db(self):
@@ -357,7 +361,6 @@ class FatalResponse:
     def __init__(self, excinfo=None):
         self.excinfo = excinfo
 
-
 def set_default_indexes(db):
     PYPI = "root/pypi"
     if not db.index_exists(PYPI):
@@ -365,3 +368,4 @@ def set_default_indexes(db):
             db.user_create("root", password="")
         db.index_create(PYPI, bases=(), type="mirror", volatile=False)
         print("set root/pypi default index")
+
