@@ -2,7 +2,7 @@ import mock
 import pytest
 
 from devpi_server.extpypi import *
-from devpi_server.main import Fatal
+from devpi_server.main import Fatal, PYPIURL_XMLRPC
 
 class TestIndexParsing:
     simplepy = URL("http://pypi.python.org/simple/py/")
@@ -30,6 +30,14 @@ class TestIndexParsing:
         """)
         assert len(result.releaselinks) == 3
 
+    def test_parse_index_simple_dir_egg_issue63(self):
+        simplepy = URL("http://pypi.python.org/simple/py/")
+        result = parse_index(simplepy,
+            """<a href="../../pkg/py-1.4.12.zip#md5=12ab">qwe</a>
+               <a href="../../pkg/#egg=py-dev">qwe</a>
+        """)
+        assert len(result.releaselinks) == 1
+
     def test_parse_index_egg_svnurl(self, monkeypatch):
         # strange case reported by fschulze/witsch where
         # urlparsing will yield a fragment for svn urls.
@@ -449,52 +457,38 @@ class TestRefreshManager:
         assert not extdb.name2serials
         extdb.mock_simple("pytest", '<a href="pytest-2.3.tgz"/a>',
                           pypiserial=20)
-        assert len(extdb.name2serials) == 1
+        extdb.mock_simple("Django", '<a href="Django-1.6.tgz"/a>',
+                          pypiserial=11)
+        assert len(extdb.name2serials) == 2
         assert len(extdb.getreleaselinks("pytest")) == 1
+        assert len(extdb.getreleaselinks("Django")) == 1
         extdb.process_changelog([
-            ["Django", "1.4", 12123, 'new release', 11],
+            ["Django", "1.4", 12123, 'new release', 25],
             ["pytest", "2.4", 121231, 'new release', 27]
         ])
         assert len(extdb.name2serials) == 2
         assert keyfs.PYPISERIALS.get()["pytest"] == 27
-        assert keyfs.PYPISERIALS.get()["Django"] == 11
+        assert keyfs.PYPISERIALS.get()["Django"] == 25
         extdb.mock_simple("pytest", '<a href="pytest-2.4.tgz"/a>',
                           pypiserial=27)
+        extdb.mock_simple("Django", '<a href="Django-1.7.tgz"/a>',
+                          pypiserial=25)
         extdb.process_refreshes()
         assert extdb.getreleaselinks("pytest")[0].basename == "pytest-2.4.tgz"
+        assert extdb.getreleaselinks("Django")[0].basename == "Django-1.7.tgz"
 
-    @pytest.fixture(params=["protocol", "socket"])
-    def raise_error(self, request):
-        import socket
-        try:
-            from xmlrpc.client import ProtocolError
-        except ImportError:
-            # PY2
-            from xmlrpclib import ProtocolError
-        if request.param == "protocol":
-            exc = ProtocolError("http://pypi.python.org/pypi", 503, "", {})
-        else:
-            exc = socket.error(111)
-        def raise_error():
-            raise exc
-        return raise_error
-
-    def test_changelog_since_serial_nonetwork(self, extdb,
-                    raise_error, monkeypatch, caplog):
+    def test_changelog_since_serial_nonetwork(self, extdb, caplog, reqmock):
         extdb.mock_simple("pytest", pypiserial=10)
-        got = []
-        def raise_xmlrpcish(since_int):
-            got.append(since_int)
-            raise_error()
-        serverproxy = mock.Mock()
-        serverproxy.changelog_since_serial.side_effect = raise_xmlrpcish
-        xmlproxy = XMLProxy(serverproxy)
+        reqreply = reqmock.mockresponse(PYPIURL_XMLRPC, code=400)
+        xmlproxy = XMLProxy(PYPIURL_XMLRPC)
         with pytest.raises(ValueError):
             extdb.spawned_pypichanges(xmlproxy, proxysleep=raise_ValueError)
         with pytest.raises(ValueError):
             extdb.spawned_pypichanges(xmlproxy, proxysleep=raise_ValueError)
-        assert got == [10,10]
-        assert caplog.getrecords(".*since_serial.*error.*")
+        calls = reqreply.requests
+        assert len(calls) == 2
+        assert xmlrpc.loads(calls[0].body) == ((10,), "changelog_since_serial")
+        assert caplog.getrecords(".*changelog_since_serial.*")
 
     def test_changelog_list_packages_no_network(self, makexom):
         xmlproxy = mock.create_autospec(XMLProxy)
